/*
 * Copyright (C) 2022 favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package assistant.app;

import assistant.Assistant;
import assistant.AssistantPanel;
import assistant.AssistantXml;
import i18n.I18N;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.HeadlessException;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.UIDefaults;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.plaf.nimbus.NimbusLookAndFeel;
import org.miginfocom.swing.MigLayout;
import resources.icons.IconUtil;
import storybook.toolkit.LOG;

/**
 *
 * @author favdb
 */
public class AssistantApp extends JDialog implements ActionListener {

	private static final String TT = "App";

	private static boolean btrace = false;
	private static boolean bdev;
	private static String filename = "";
	//private static JFrame frame;
	private static boolean subJar = false;
	private static AssistantApp app;

	public static boolean isTrace() {
		return btrace;
	}

	public static boolean isDev() {
		return bdev;
	}

	private static Font fontDefault;

	public static Font getFontDefault() {
		return fontDefault;
	}

	private static void initLAF() {
		try {
			//UIDefaults defaults = UIManager.getDefaults();
			Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
			int sz = (int) d.getWidth() / 800 * 10;
			for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
				if ("Nimbus".equals(info.getName())) {
					UIManager.setLookAndFeel(new NimbusLookAndFeel() {

						@Override
						public UIDefaults getDefaults() {
							UIDefaults ret = super.getDefaults();
							ret.put("defaultFont",
									new Font(Font.SANS_SERIF, Font.PLAIN, sz)); // supersize me
							return ret;
						}

					});
					break;
				}
			}
		} catch (HeadlessException | UnsupportedLookAndFeelException ex) {
			LOG.err(TT + ".initLAF()", ex);
		}
	}
	private static AssistantPref preferences;

	public static AssistantPref getPref() {
		return preferences;
	}

	private AssistantPanel dataPanel;
	private boolean modified = false;
	private AssistantXml assistantXml;

	public AssistantPanel getDataPanel() {
		return dataPanel;
	}

	public static void main(String args[]) {
		for (String str : args) {
			switch (str) {
				case "--sub":
					subJar = true;
					break;
				case "--dev":
					btrace = true;
					bdev = true;
					break;
				case "--trace":
					btrace = true;
					break;
				default:
					File f = new File(str);
					if (f.exists() && str.toLowerCase().endsWith("xml")) {
						filename = str;
					}
					break;
			}
		}
		if (btrace) {
			LOG.setTrace();
		}
		initLAF();
		java.awt.EventQueue.invokeLater(() -> {
			app = new AssistantApp();
			app.initialize();
			app.setVisible(true);
		});
	}

	public static AssistantApp getInstance() {
		return app;
	}

	private AssitantAction appAction;

	public AssitantAction getAppAction() {
		return appAction;
	}

	private AssistantMenu appMenu;

	public AssistantMenu getAppMenu() {
		return appMenu;
	}

	public AssistantApp() {
		this.setModal(true);
		// empty
	}

	public void initialize() {
		init();
		initUi();
	}

	public void init() {
		//LOG.trace(TT + ".init()");
		File file = null;
		if (!filename.isEmpty()) {
			file = new File(filename);
			if (!file.exists()) {
				file = null;
			}
		}
		IconUtil.setDefSize();
		Assistant.init(file);
		appAction = new AssitantAction(this);
		appMenu = new AssistantMenu(this);
		appMenu.setSaveEnable(false);
		setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
		addWindowListener(new MainFrameWindowAdaptor());
		I18N.initMsgProp();
		setName("mainDialog");
		setLayout(new MigLayout());
		setJMenuBar(appMenu);
		dataPanel = new AssistantPanel(this, true, null);
	}

	public void initUi() {
		//LOG.trace(TT + ".initUi()");
		setAppTitle();
		dataPanel.reinit();
		add(dataPanel);
		this.pack();
		this.setLocationRelativeTo(null);
	}

	public AssistantPref getPreferences() {
		return preferences;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		// empty
	}

	public void setModified(boolean b) {
		//LOG.trace(TT + ".setModified(b=" + (b ? "true" : "false") + ")");
		modified = b;
		setAppTitle();
		appMenu.setSaveEnable(b);
	}

	public boolean isModified() {
		return modified;
	}

	public void setAppTitle() {
		String star = (modified ? "*" : "");
		this.setTitle(star + I18N.getMsg("assistant", Assistant.getLastOpened()) + star);
	}

	public void close() {
		//LOG.trace(TT + ".close()");
		if (modified) {
			Object[] choix = {
				I18N.getMsg("cancel"),
				I18N.getMsg("file.save"),
				I18N.getMsg("ignore")
			};
			int n = JOptionPane.showOptionDialog(null,
					I18N.getMsg("exit.confirm"),
					I18N.getMsg("exit"),
					JOptionPane.YES_NO_OPTION,
					JOptionPane.QUESTION_MESSAGE,
					null, choix, choix[0]);
			if (n == 0) {
				return;
			}
			if (n == 1) {
				if (!appAction.fileSave()) {
					return;
				}
			}
		}
		if (subJar) {
			dispose();
		} else {
			System.exit(0);
		}
	}

	public void refreshAll() {
		dataPanel.reinit();
	}

	private class MainFrameWindowAdaptor extends WindowAdapter {

		@Override
		public void windowClosing(WindowEvent evt) {
			AssistantApp.this.close();
		}

		@Override
		public void windowOpened(WindowEvent e) {
			// empty
		}

		@Override
		public void windowClosed(WindowEvent e) {
			// empty
		}

		@Override
		public void windowIconified(WindowEvent e) {
			// empty
		}

		@Override
		public void windowDeiconified(WindowEvent e) {
			// empty
		}

		@Override
		public void windowActivated(WindowEvent e) {
			// empty
		}

		@Override
		public void windowDeactivated(WindowEvent e) {
			// empty
		}
	}

}
