/*
 * Copyright (C) 2018 FaVdB
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a fileCopy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.exim.exporter;

import i18n.I18N;
import java.awt.HeadlessException;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.util.List;
import javax.swing.JOptionPane;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;
import resources.icons.IconUtil;
import storybook.Const;
import storybook.model.EntityUtil;
import storybook.model.book.Book;
import storybook.model.book.BookParamExport;
import storybook.model.hbn.entity.Chapter;
import storybook.model.hbn.entity.Endnote;
import storybook.model.hbn.entity.Item;
import storybook.model.hbn.entity.Location;
import storybook.model.hbn.entity.Part;
import storybook.model.hbn.entity.Person;
import storybook.model.hbn.entity.Scene;
import storybook.model.hbn.entity.Strand;
import storybook.toolkit.DateUtil;
import storybook.toolkit.LOG;
import storybook.toolkit.ListUtil;
import storybook.toolkit.Markdown;
import storybook.toolkit.StringUtil;
import storybook.toolkit.TextTransfer;
import storybook.toolkit.html.Html;
import storybook.ui.MainFrame;

/**
 * Export the whole book to an HTML file (or multifile, or String)
 *
 * @author FaVdB
 */
public class ExportBookToHtml extends AbstractExport {

	private static final String TT = "ExportBookToHtml",
			INDENT = "&nbsp;&nbsp;&nbsp;&nbsp;",
			ENDNOTEFILE = "endnote.html";
	private BookParamExport paramBook;
	private Chapter chapterFirst, chapterLast, chapterPrior, chapterNext, chapterCur;
	private Scene scenePrior, sceneNext;
	private StringBuilder buffer;
	private boolean bSeparator;
	private String sceneSeparator = Const.SCENE_SEPARATOR;
	private Strand strand;
	private List<Part> parts;
	private List<Chapter> chapters;
	private List<Scene> scenes;
	private Part onlyPart = null;
	private boolean tocLink = false;
	private List<Endnote> endnotes;

	enum NAV {
		NAV_BACKWARD,
		NAV_BACKWARD_GRAYED,
		NAV_FORWARD,
		NAV_FORWARD_GRAYED,
		AR_LEFT,
		AR_LEFT_GRAYED,
		AR_RIGHT,
		AR_RIGHT_GRAYED,
		NAV_FIRST,
		NAV_FIRST_GRAYED,
		NAV_PREV,
		NAV_PREV_GRAYED,
		NAV_NEXT,
		NAV_NEXT_GRAYED,
		NAV_LAST,
		NAV_LAST_GRAYED,
		SUMMARY;

		@Override
		public String toString() {
			return name().toLowerCase();
		}
	};

	/**
	 * ExportBookToHtml class
	 *
	 * @param m
	 */
	public ExportBookToHtml(MainFrame m) {
		super(m, "html");
		init();
	}

	/**
	 * initialize
	 *
	 */
	@SuppressWarnings("unchecked")
	private void init() {
		//App.trace(TT + ".init()");
		paramBook = mainFrame.getBook().getParamExport();
		sceneSeparator = mainFrame.getBook().getParamLayout().getSceneSeparatorValue();
		parts = EntityUtil.findEntities(mainFrame, Book.TYPE.PART);
		chapters = EntityUtil.findEntities(mainFrame, Book.TYPE.CHAPTER);
		chapterFirst = chapters.get(0);
		chapterLast = chapters.get(chapters.size() - 1);
		scenes = EntityUtil.findEntities(mainFrame, Book.TYPE.SCENE);
		endnotes = (List<Endnote>) EntityUtil.findEntities(mainFrame, Book.TYPE.ENDNOTE);
	}

	/**
	 * do an export to a file
	 *
	 * @param m
	 * @return true if done
	 */
	public static boolean toFile(MainFrame m) {
		//App.trace(TT + ".toFile(mainFrame)");
		ExportBookToHtml export = new ExportBookToHtml(m);
		return export.writeFile();
	}

	/**
	 * do an export to a String
	 *
	 * @param m
	 * @return true if done
	 */
	public static String toString(MainFrame m) {
		//App.trace(TT + ".toString(mainFrame)");
		ExportBookToHtml export = new ExportBookToHtml(m);
		return export.getString();
	}

	/**
	 * do an export to clipboard
	 *
	 * @param mainFrame
	 * @param onlyPart
	 */
	public static void toClipboard(MainFrame mainFrame, Part onlyPart) {
		//App.trace(TT + ".toClipboarb(mainFrame," + (onlyPart == null ? "nul" : onlyPart.getName()) + ")");
		ExportBookToHtml export = new ExportBookToHtml(mainFrame);
		export.tocLink = false;
		export.onlyPart = onlyPart;
		export.param.setFormat(F_HTML);
		export.paramBook.setHtmlMultiChapter(false);
		export.paramBook.setHtmlMultiScene(false);
		export.paramBook.setHighlight(Html.EM_LEFTASIS);
		try {
			TextTransfer tf = new TextTransfer();
			String strText = export.getString()
					.replaceAll("<a name=.*</a>", "");
			tf.setClipboardContents(strText);
			JOptionPane.showMessageDialog(mainFrame,
					I18N.getMsg("book.copy.text.confirmation"),
					I18N.getMsg("copied.title"), 1);
		} catch (HeadlessException exc) {
		}
	}

	/**
	 * do an export to a String for a JPanel content
	 *
	 * @param mainFrame
	 * @param strand
	 * @param onlyPart
	 * @return
	 */
	public static String toPanel(MainFrame mainFrame, Strand strand, Part onlyPart) {
		/*App.trace(TT+".toPanel(m, strand=" + (strand == null ? "null" : strand.toString())
				+ ",onlyCurrentPart=" + (onlyCurrentPart ? "true" : "false")
				+ "tocLinks=" + (tocLinks ? "true" : "false")
				+ ")");*/
		ExportBookToHtml exp = new ExportBookToHtml(mainFrame);
		exp.strand = strand;
		exp.param.setFormat(F_HTML);
		exp.paramBook.setHtmlMultiChapter(false);
		exp.paramBook.setHtmlMultiScene(false);
		exp.paramBook.setHtmlNav(false);
		exp.tocLink = true;
		exp.onlyPart = onlyPart;
		StringBuilder lbuf = new StringBuilder();
		lbuf.append(exp.tocGet());
		exp.bSeparator = true;
		lbuf.append(exp.getBody());
		return (lbuf.toString());
	}

	/**
	 * write a write
	 *
	 * @return
	 */
	public boolean writeFile() {
		//App.trace(TT + ".writeFile()");
		String fileName = mainFrame.getH2File().getName();
		String dir = paramBook.getDirectory() + File.separator + "img";
		if (param.isHtml()) {
			File f = new File(dir);
			if (!f.exists() || f.isDirectory()) {
				f.mkdir();
			}
			if (paramBook.isMulti()) {
				for (NAV nav : NAV.values()) {
					copyImage(nav.toString(), dir);
				}
				fileName = "index";
			}
		}
		if (askFileExists(fileName) && JOptionPane.showConfirmDialog(mainFrame,
				I18N.getMsg("export.replace", param.getFileName()),
				I18N.getMsg("export"),
				JOptionPane.OK_CANCEL_OPTION) == JOptionPane.CANCEL_OPTION) {
			return (false);
		}
		paramBook = mainFrame.getBook().getParamExport();
		if (!openFile(fileName)) {
			return (false);
		}
		writeText(ExportBook.getTitle(book));
		writeText(Html.intoP(Html.intoI(book.getBlurb())));
		tocGet();
		bSeparator = true;
		getBody();
		closeFile(SILENT);
		JOptionPane.showMessageDialog(mainFrame,
				I18N.getMsg("export.success", paramBook.getDirectory()),
				I18N.getMsg("export"), JOptionPane.INFORMATION_MESSAGE);
		return true;
	}

	/**
	 * get the Book as a HTML String
	 *
	 * @return
	 */
	public String getString() {
		//App.trace(TT + ".getString()");
		StringBuilder buf = new StringBuilder();
		param.setHtmlMultiChapter(false);
		param.setHtmlMultiScene(false);
		param.setHtmlNav(false);
		buf.append(Html.DOCTYPE).append(Html.HTML_B).append(Html.BODY_B);
		buf.append(ExportBook.getTitle(book));
		bSeparator = true;
		buf.append(getBody());
		buf.append(Html.BODY_E).append(Html.HTML_E);
		return buf.toString();
	}

	/**
	 * check if table of content is needed
	 *
	 * @return
	 */
	private boolean tocIs() {
		return param.getLayout().getPartTitle()
				|| param.getLayout().getChapterTitle()
				|| param.getLayout().getSceneTitle()
				&& (parts.size() > 1 || chapters.size() > 1 || scenes.size() > 0);
	}

	/**
	 * get the toc of the Book
	 *
	 * @return
	 */
	private String tocGet() {
		//App.trace(TT + ".getTocBook()");
		StringBuilder buf = new StringBuilder();
		if (!tocIs()) {
			return ("<a name=\"toc\"></a>");
		}
		if (!scenes.isEmpty()
				&& (param.getLayout().getPartTitle()
				|| param.getLayout().getChapterTitle()
				|| param.getLayout().getSceneTitle())) {
			buf.append("<p><a name=\"toc\" /><b>")
					.append(I18N.getMsg("export.toc")).append("</b>");
		}
		for (Part part : parts) {
			if (onlyPart != null && !part.equals(onlyPart)) {
				continue;
			}
			if (param.getLayout().getPartTitle()) {
				buf.append("<br>").append(tocPart(part));
			}
			for (Chapter chapter : chapters) {
				buf.append(tocChapter(part, chapter));
			}
		}
		if (this.onlyPart == null && endnotesHas()) {
			buf.append(Html.intoH(3, endnotesLinkTo(0)));
		}
		buf.append(Html.P_E);
		if (isOpened && paramBook.isMulti()) {
			chapterNext = chapters.get(0);
			sceneNext = scenes.get(0);
			buf.append(navbar());
			writeText(buf.toString());
			writeText(Html.BODY_E);
			writeText(Html.HTML_E);
			closeFile(SILENT);
			return "";
		} else {
			writeText(buf.toString());
		}
		return buf.toString();
	}

	private String tocTo(int level, String number, String string) {
		StringBuilder b = new StringBuilder();
		b.append("<br>\n");
		for (int i = 0; i < level; i++) {
			b.append("&emsp;");
		}
		b.append("<a")
				.append(" name=\"t").append(number).append("\"")
				.append(" href=\"#").append(number).append("\"")
				.append(">")
				.append(string).append("</a>");
		return b.toString();
	}

	private String titleOf(int level, String number, String name) {
		if (level == 0) {
			return Html.intoA(number, "", "");
		}
		return "<h" + level + ">" + Html.intoA(number, "", name) + "</h" + level + ">\n";
	}

	/**
	 * get the Part table of content
	 *
	 * @param part
	 * @return
	 */
	public String tocPart(Part part) {
		if (param.getLayout().getPartTitle()) {
			return tocTo(1, "P" + part.getId().toString(), part.getName());
		} else {
			return "";
		}
	}

	public String titlePart(Part part) {
		if (param.getLayout().getPartTitle()) {
			return titleOf(1, part.getIdent(), part.getName());
		} else {
			return "";
		}
	}

	/**
	 * get the Chapter table of content
	 *
	 * @param part
	 * @param chapter
	 * @return
	 */
	public String tocChapter(Part part, Chapter chapter) {
		int n = 1;
		if (param.getLayout().getPartTitle()) {
			n++;
		}
		StringBuilder b = new StringBuilder();
		if (param.getLayout().getChapterTitle()) {
			b.append(tocTo(n, chapter.getIdent(), chapterGetTitle(chapter)));
		}
		for (Scene scene : EntityUtil.findScenes(mainFrame, chapter)) {
			b.append(tocScene(scene));
		}
		return b.toString();
	}

	private String tocScene(Scene scene) {
		if (paramBook.isMulti() || !paramBook.getLayout().getSceneTitle()) {
			return "";
		}
		int n = 2;
		if (param.getLayout().getPartTitle()) {
			n++;
		}
		if (param.getLayout().getChapterTitle()) {
			n++;
		}
		if (sceneIsOK(scene)) {
			return tocTo(n, scene.getIdent(), scene.getName());
		}
		return "";
	}

	/**
	 * get a link to a Scene
	 *
	 * @param scene
	 * @param img
	 * @return
	 */
	public String sceneLinkTo(Scene scene, String img) {
		/*App.trace(TT+".linkToScene(scene="+App.traceEntity(scene)+
			", img="+(img==null?"null":img)+")");*/
		if (scene == null) {
			return "scene empty and img empty";
		}
		String image = scene.getName();
		if (img != null && !img.isEmpty()) {
			image = Html.intoIMG("img/" + img + ".png", image, image);
		}
		if (isOpened && paramBook.isMulti()) {
			return String.format("<a href=\"%s\">%s</a>", sceneGetFilename(scene), image);
		}
		return String.format("<a href=\"#%s\">%s</a>", scene.getIdent(), image);
	}

	/**
	 * get a link to a Chapter
	 *
	 * @param chapter
	 * @param img
	 * @return
	 */
	private String chapterLinkTo(Chapter chapter, String img) {
		/*App.trace(TT+".linkToChapter(chapter="+
			(chapter!=null?chapter.getTitle():"null")+
			", img="+(img==null?"null":img)+")");*/
		if (chapter == null) {
			return Html.intoIMG("img/" + img + ".png", "", "");
		}
		String title = "";
		if (param.getLayout().getChapterNumber()) {
			if (param.getLayout().getChapterRoman()) {
				title = (String) StringUtil.intToRoman(chapter.getChapterno()) + " ";
			} else {
				title = chapter.getChapterno().toString() + " ";
			}
		}
		if (param.getLayout().getChapterTitle()) {
			title += chapter.getName();
		}
		if (img != null && !img.isEmpty() && paramBook.getHtmlNavImage()) {
			title = Html.intoIMG("img/" + img + ".png", title, title);
		}
		if (isOpened && paramBook.isMulti()) {
			return (String.format("<a href=\"%s\">%s</a>", chapterGetFilename(chapter), title));
		}
		return String.format("<a href=\"#C%s\">%s</a>", chapter.getIdent(), title);
	}

	/**
	 * get a link to an Endnote
	 *
	 * @param n
	 * @param img
	 * @return
	 */
	private String endnotesLinkTo(int n, String... img) {
		//App.trace(TT+".linkToEndnotes()");
		if (endnotes.size() < 1) {
			return "";
		}
		String title = I18N.getMsg("endnotes");
		String link = "#endnotes";
		String number = String.format("%03d", n);
		if (n == 0) {
			number = "";
		} else {
			number = "#" + number;
		}
		if (isOpened && paramBook.isMulti()) {
			if (img != null && img.length > 0) {
				title = Html.intoIMG(img[0], title, title);
			}
			link = ENDNOTEFILE;
		}
		return Html.intoA("", link + number, title);
	}

	/**
	 * get the HTML body (includes parts, chapters and scenes)
	 *
	 * @return a String representation of the HTML code
	 */
	public String getBody() {
		//App.trace(TT + ".getBody()");
		buffer = new StringBuilder();
		for (Part part : parts) {
			if (onlyPart != null && !part.equals(onlyPart)) {
				continue;
			}
			buffer.append(partGetTitle(part));
			for (Chapter chapter : chapters) {
				if (chapter.hasPart() && chapter.getPart().equals(part)) {
					chapterGet(chapter);
				}
			}
		}
		if (isOpened && paramBook.isMulti()) {
			buffer.append(navbar());
		}
		writeText(buffer.toString());
		if (onlyPart == null && endnotesHas()) {
			if (paramBook.isMulti()) {
				openFile(ENDNOTEFILE.replace(Html.EXT, ""));
				buffer = new StringBuilder();
			}
			buffer.append(endnotesGet());
			if (tocLink) {
				buffer.append(tocGetLink());
			} else {
				if (paramBook.isMulti()) {
					buffer.append(navbar());
				}
			}
			writeText(buffer.toString());
		}
		return buffer.toString();
	}

	/**
	 * get a Chapter in HTML code
	 *
	 * @param chapter
	 */
	private void chapterGet(Chapter chapter) {
		//App.trace(TT + ".chapterGet(chapter=" + App.traceEntity(chapter) + ")");
		chapterStart(chapter);
		int lev = 1;
		if (param.getLayout().getPartTitle()) {
			lev++;
		}
		if (param.getLayout().getChapterTitle()) {
			buffer.append(titleOf(lev, chapter.getIdent(), chapterGetTitle(chapter)));
		}
		buffer.append(chapterGetDidascalie(chapter));
		buffer.append(chapterGetDescription(chapter));
		for (Scene scene : EntityUtil.findScenes(mainFrame, chapter)) {
			if (sceneIsOK(scene)) {
				sceneGet(scene);
			}
		}
		chapterEnd();
	}

	/**
	 * get the starting Chapter context, open the file if it's needed
	 *
	 * @param chapter
	 */
	private void chapterStart(Chapter chapter) {
		if (paramBook.getHtmlMultiChapter()) {
			openFile(chapterGetFilename(chapter).replace(Html.EXT, ""));
		}
		chapterCur = chapter;
		chapterPrior = chapterFindPrior(chapter);
		chapterNext = chapterFindNext(chapter);
		if (buffer == null) {
			buffer = new StringBuilder();
		}
	}

	/**
	 * do the ending action for the current Chapter
	 */
	private void chapterEnd() {
		if (paramBook.getHtmlMultiChapter() && buffer.length() > 0) {
			buffer.append(Html.P_E);
			buffer.append(navbar());
			writeText(buffer.toString());
			closeFile(SILENT);
			buffer = new StringBuilder();
		}
	}

	/**
	 * find the prior chapter
	 *
	 * @param chapter
	 * @return null if there is no prior chapter
	 */
	private Chapter chapterFindPrior(Chapter chapter) {
		for (int i = 0; i < scenes.size(); i++) {
			Chapter s = chapters.get(i);
			if (s.getId().equals(chapter.getId())) {
				if (i > 0) {
					return chapters.get(i - 1);
				}
				break;
			}
		}
		return null;
	}

	/**
	 * find the next chapter
	 *
	 * @param chapter
	 * @return null if there is no next chapter
	 */
	private Chapter chapterFindNext(Chapter chapter) {
		for (int i = 0; i < chapters.size(); i++) {
			Chapter s = chapters.get(i);
			if (s.getId().equals(chapter.getId())) {
				if (i < chapters.size() - 1) {
					return chapters.get(i + 1);
				}
				break;
			}
		}
		return null;
	}

	/**
	 * check for Scene possible insertion
	 *
	 * @param scene
	 * @return true if the Scene would be inserted
	 */
	private boolean sceneIsOK(Scene scene) {
		return !(scene.getInformative()
				|| (strand != null
				&& (!strand.equals(scene.getStrand())
				&& !scene.getStrands().contains(strand))));
	}

	/**
	 * get the Scene HTML code
	 *
	 * @param scene
	 */
	private void sceneGet(Scene scene) {
		//LOG.trace(TT + ".sceneGet(scene=" + LOG.traceEntity(scene) + ")");
		sceneStart(scene);
		if (param.getLayout().getSceneTitle()) {
			int n = 2;
			if (param.getLayout().getPartTitle()) {
				n++;
			}
			if (param.getLayout().getChapterTitle()) {
				n++;
			}
			buffer.append(titleOf(n, scene.getIdent(), scene.getName()));
		}
		if (param.getLayout().getSceneDidascalie()) {
			buffer.append(sceneGetDidascalie(param, scene));
		}
		String x = scene.getTextToHtml(paramBook.isMulti());
		if (param.getFileName() != null && !param.getFileName().isEmpty()) {
			x = Html.setToRelative(x, mainFrame.getH2File().getPath() + File.separator);
		}
		if (book.isMarkdown()) {
			Markdown md = new Markdown("ExportBook", "text/plain", "");
			md.setHeader(scene, book.info.scenarioGet());
			md.setText(scene.getTextToHtml(paramBook.getHtmlMultiScene()));
			x = md.getHtmlBody();
		} else {
			x = Html.emTag(x, paramBook.getHighlight());
		}
		if (x.contains("#chapscene")) {
			x = linkReplaceInternal(x);
		}
		buffer.append(x).append("\n");
		if (tocIs() && !isOpened) {
			buffer.append(tocGetLink());
		}
		if (param.getLayout().getSceneSeparator() && bSeparator) {
			buffer.append(Html.P_CENTER).append(sceneSeparator).append("</p>\n");
		}
		sceneEnd();
	}

	/**
	 * find the prior Scene
	 *
	 * @param scene
	 * @return null if there is no prior Scene
	 */
	private Scene sceneFindPrior(Scene scene) {
		for (int i = 0; i < scenes.size(); i++) {
			Scene s = scenes.get(i);
			if (s.getId().equals(scene.getId())) {
				if (i > 0) {
					return scenes.get(i - 1);
				}
				break;
			}
		}
		return null;
	}

	/**
	 * find the next Scene
	 *
	 * @param scene
	 * @return null if there is no next Scene
	 */
	private Scene sceneFindNext(Scene scene) {
		for (int i = 0; i < scenes.size(); i++) {
			Scene s = scenes.get(i);
			if (s.getId().equals(scene.getId())) {
				if (i < scenes.size() - 1) {
					return scenes.get(i + 1);
				}
				break;
			}
		}
		return null;
	}

	/**
	 * get the starting Scene context, open the file if it's needed
	 *
	 * @param scene
	 */
	private void sceneStart(Scene scene) {
		if (paramBook.getHtmlMultiScene()) {
			openFile(sceneGetFilename(scene).replace(Html.EXT, ""));
		}
		scenePrior = sceneFindPrior(scene);
		sceneNext = sceneFindNext(scene);
		if (buffer == null) {
			buffer = new StringBuilder();
		}
	}

	/**
	 * do the endind action for the current Scene
	 */
	private void sceneEnd() {
		if (paramBook.getHtmlMultiScene() && buffer.length() > 0) {
			buffer.append(navbar());
			writeText(buffer.toString());
			closeFile(SILENT);
			buffer = new StringBuilder();
		}
	}

	/**
	 * get the link to toc
	 *
	 * @return
	 */
	public String tocGetLink() {
		//return Html.intoP(
		//		Html.intoA("", "#toc", I18N.getMsg("toc")),
		//		Html.FONT_SIZE + ":8px;" + Html.ALIGN_LEFT);
		String img = "file://" + mainFrame.getImageDir() + "/summary.png";
		if (isOpened) {
			img = "img/summary.png";
		}
		StringBuilder b = new StringBuilder();
		b.append("<table "
				+ "BORDER=\"0\" "
				+ "CELLSPACING=\"0\" "
				+ "CELLSPACING=\"0\" "
				+ "style=\"width:100%;background-color: rgb(220, 220, 220);"
				+ "\">").append(Html.TR_B);
		b.append(Html.intoTD(
				Html.intoA("", "#toc", Html.intoIMG(img, "", I18N.getMsg("toc"), 20)),
				Html.FONT_SIZE + ":8px;" + Html.AL_LEFT, "text-align:center;padding:0px;"
		//)
		));
		b.append(Html.TR_E).append(Html.TABLE_E);
		return b.toString();
	}

	/**
	 * get the Endnotes
	 *
	 * @return the String text of the HTML code
	 */
	public String endnotesGet() {
		StringBuilder buf = new StringBuilder();
		buf.append(Html.intoA("endnotes", "", ""));
		buf.append(Html.intoH(1, I18N.getMsg("endnote")));
		for (Object obj : endnotes) {
			Endnote endnote = (Endnote) obj;
			if (endnote.getScene() == null) {
				LOG.err(TT + ".getNotes endnote n° " + endnote.getNumber() + " the scene is null");
				continue;
			}
			String link = endnote.getLinkToScene(sceneGetFilename(endnote.getScene()));
			String text = endnote.getNotes();
			if (text.startsWith(Html.P_B)) {
				text = Html.P_B + link + text.substring(3);
			} else {
				text = Html.P_B + link + " " + text + Html.P_E;
			}
			buf.append(text).append("\n");
		}
		if (paramBook.getHtmlMultiScene()) {
			writeText(buf.toString());
			closeFile(SILENT);
			buf = new StringBuilder();
		}
		return (buf.toString());
	}

	/**
	 * get the didascalie of a Scene
	 *
	 * @param param
	 * @param scene
	 * @return a String reprenting the HTML code
	 */
	public static String sceneGetDidascalie(BookParamExport param, Scene scene) {
		//App.trace(TT+".getDidascalie("+scene.getTitle()+")");
		String rc = "";
		if (param.getLayout().getSceneDidascalie()) {
			if (!scene.getPersons().isEmpty()) {
				rc += "<i>" + Html.intoB(I18N.getMsg("persons")) + " : ";
				for (Person p : scene.getPersons()) {
					rc += p.getFullName() + ", ";
				}
				rc = rc.substring(0, rc.length() - 2);
				rc += "</i>" + Html.BR;
			}
			if (!scene.getLocations().isEmpty()) {
				rc += "<i>" + Html.intoB(I18N.getMsg("locations")) + " : ";
				for (Location p : scene.getLocations()) {
					rc += p.getFullName() + ", ";
				}
				rc = rc.substring(0, rc.length() - 2);
				rc += "</i>" + Html.BR;
			}
			if (!scene.getItems().isEmpty()) {
				rc += "<i>" + Html.intoB(I18N.getMsg("items")) + " : ";
				for (Item p : scene.getItems()) {
					rc += p.getName() + ", ";
				}
				rc = rc.substring(0, rc.length() - 2);
				rc += "</i>" + Html.BR;
			}
			if (!scene.getScenario_pitch().isEmpty()) {
				rc += "<i>" + Html.intoB(I18N.getMsg("scenario.pitch")) + " : ";
				rc += scene.getScenario_pitch();
				rc += "</i>" + Html.BR;
			}
			if (!rc.isEmpty()) {
				rc = Html.intoP(rc, "text-align:right");
			}
		}
		return (rc);
	}

	/**
	 * replace internal link to real link
	 *
	 * @param x: String for the text to parse
	 *
	 * @return the replacing String
	 */
	private String linkReplaceInternal(String x) {
		//App.trace(TT+".linkReplaceInternal("+TextUtil.ellipsize(x, 32)+")");
		if (!x.contains("#chapscene")) {
			return (x);
		}
		Document doc = Jsoup.parse(x);
		Elements links = doc.select("a");
		for (Element e : links) {
			if (e.attr("href").contains("#chapscene")) {
				String r = e.attr("href").replace("#chapscene", "");
				String[] y = r.split("\\.");
				if (y.length < 1) {
					continue;
				}
				String chap = y[0];
				String scene = "";
				if (y.length > 1) {
					scene = chap + "." + y[1];
				}
				if ("".equals(chap)) {
					continue;
				}
				if (chap.length() < 2) {
					chap = "0" + chap;
				}
				if (paramBook.getHtmlMultiChapter()) {
					r = book.getTitle() + "-C" + chap + Html.EXT;
					if (!"".equals(scene)) {
						r += "#" + scene;
					}
				} else if (paramBook.getHtmlMultiScene()) {
					r = book.getTitle() + "-S" + r + Html.EXT;
				} else {
					r = "#" + r;
				}
				e.attr("href", r);
			}
		}
		String z = (doc.select("body").get(0).children().toString());
		return (z);
	}

	/**
	 * get nav bar for the current Scene
	 *
	 * @return
	 */
	private String sceneNavbar() {
		StringBuilder buf = new StringBuilder();
		buf.append("<p style=\"text-align: right;\">");
		String link = "";
		// to prior scene
		if (scenePrior != null) {
			link = sceneLinkTo(scenePrior, NAV.AR_LEFT.toString());
		} else {
			if (paramBook.getHtmlNavImage()) {
				link = Html.intoIMG("img/" + NAV.AR_LEFT_GRAYED + ".png", "", "");
			}
		}
		buf.append(link);

		// to next scene
		if (sceneNext != null) {
			link = sceneLinkTo(sceneNext, NAV.AR_RIGHT.toString());
		} else {
			link = Html.intoIMG("img/" + NAV.AR_RIGHT_GRAYED + ".png", "", "");
		}
		buf.append(link);
		return buf.toString();
	}

	/**
	 * get nav bar for chapter and/or scene
	 *
	 * @return
	 */
	private String navbar() {
		if (!paramBook.getHtmlNav()) {
			return "";
		}
		StringBuilder b = new StringBuilder();
		b.append("<table style=\"width:100%; background-color: lightgray;\">\n");
		b.append(Html.TR_B);
		// to summary
		String toc = I18N.getMsg("export.nav.summary");
		String link = Html.intoIMG("img/" + NAV.SUMMARY + ".png", toc, toc);
		if (paramBook.getHtmlNavImage()) {
			link = Html.intoA("", "index.html", link);
		} else {
			link = Html.intoA("", "index.html", toc);
		}
		b.append(Html.intoTD(link));

		if (paramBook.getHtmlMultiChapter()) {
			b.append(Html.intoTD(chapterNavbar(), "align=\"right\""));
		}
		if (paramBook.getHtmlMultiScene()) {
			b.append(Html.intoTD(sceneNavbar(), "align=\"right\""));
		}
		b.append(Html.TR_E);
		b.append(Html.TABLE_E);
		return b.toString();
	}

	/**
	 * get Chapter nav bar
	 *
	 * @return
	 */
	private String chapterNavbar() {
		StringBuilder buf = new StringBuilder();
		if (!paramBook.getHtmlNav()) {
			return "";
		}
		buf.append("<p style=\"text-align: right;\">");
		String link = chapterLinkTo(null, NAV.NAV_FIRST_GRAYED.toString());
		// first chapter
		if (chapterCur != null && !chapterCur.equals(chapterFirst)) {
			link = chapterLinkTo(chapterFirst, NAV.NAV_FIRST.toString());
		}
		buf.append(link);

		// to previous chapter
		if (chapterPrior != null && !chapterCur.equals(chapterPrior)) {
			link = chapterLinkTo(chapterPrior, NAV.NAV_BACKWARD.toString());
		} else {
			link = chapterLinkTo(null, NAV.NAV_BACKWARD_GRAYED.toString());
		}
		buf.append(link);
		// to next chapter
		if (chapterNext != null && !chapterNext.equals(chapterLast)) {
			link = chapterLinkTo(chapterNext, NAV.NAV_FORWARD.toString());
		} else if (chapterCur != null && chapterCur.equals(chapterLast)) {
			link = endnotesLinkTo(0, NAV.NAV_FORWARD.toString());
		} else {
			link = chapterLinkTo(null, NAV.NAV_FORWARD_GRAYED.toString());
		}
		buf.append(link);
		// to last chapter
		if (chapterCur != null && !chapterCur.equals(chapterLast)) {
			link = chapterLinkTo(chapterLast, NAV.NAV_LAST.toString());
		} else {
			link = chapterLinkTo(null, NAV.NAV_LAST_GRAYED.toString());
		}
		buf.append(link);
		buf.append(Html.P_E);
		return (buf.toString());
	}

	/**
	 * check if endnotes is not empty
	 *
	 * @return
	 */
	private boolean endnotesHas() {
		return (endnotes != null && !endnotes.isEmpty());
	}

	/**
	 * get title of a Part
	 *
	 * @param part
	 * @return
	 */
	public String partGetTitle(Part part) {
		//App.trace(TT+".getPartTitle("+part.getName()+")");
		String buf = "";
		if (param.getLayout().getPartTitle()) {
			buf = Html.intoH(1, part.getName());
		}
		return buf;
	}

	/**
	 * get the Chapter title, if needed, with number
	 *
	 * @param chapter
	 * @return
	 */
	private String chapterGetTitle(Chapter chapter) {
		StringBuilder b = new StringBuilder();
		if (param.getLayout().getChapterTitle()) {
			if (param.getLayout().getChapterNumber()) {
				if (param.getLayout().getChapterRoman()) {
					b.append(StringUtil.intToRoman(chapter.getChapterno())).append(" ");
				} else {
					b.append(chapter.getChapterno().toString()).append(" ");
				}
			}
			b.append(chapter.getName());
		}
		return b.toString();
	}

	/**
	 * get the Chapter didascalie (dates and locations)
	 *
	 * @param chapter
	 * @return
	 */
	private String chapterGetDidascalie(Chapter chapter) {
		//App.trace(TT + ".chapterGetDateLocation(chapter=" + App.traceEntity(chapter) + ")");
		StringBuilder b = new StringBuilder();
		if (param.getLayout().getChapterDateLocation()) {
			String date = DateUtil.getNiceDates(EntityUtil.findDates(mainFrame, chapter));
			String loc = ListUtil.join(EntityUtil.findLocations(mainFrame, chapter), ", ");
			if (!(date + loc).isEmpty()) {
				StringBuilder bx = new StringBuilder();
				if (!date.isEmpty()) {
					bx.append(date);
				}
				if (!loc.isEmpty()) {
					if (!date.isEmpty()) {
						bx.append(" : ");
					}
					bx.append(loc);
				}
				b.append(Html.intoP(Html.intoI(bx.toString()), "margin-right: 3.5cm;"));
			}
		}
		return b.toString();
	}

	/**
	 * get the Chapter description
	 *
	 * @param chapter
	 * @return
	 */
	private String chapterGetDescription(Chapter chapter) {
		//App.trace(TT + ".chapterGetDescription(chapter=" + App.traceEntity(chapter) + ")");
		if (param.getLayout().getChapterDescription() && !chapter.getDescription().isEmpty()) {
			return (Html.intoP(Html.intoI(chapter.getDescription()), "margin-left: 3.5cm;text-align: right;"));
		}
		return "";
	}

	/**
	 * copy an image from resources to the export directory
	 *
	 * @param img: image source name without extent
	 * @param dir: the export directory
	 */
	private void copyImage(String img, String dir) {
		File fdir = new File(dir);
		fdir.mkdirs();
		File out = new File(dir + File.separator + img + ".png");
		if (out.exists()) {
			return;

		}
		Class<?> x = IconUtil.class;
		String path = "png/" + img.toLowerCase().replace(".", "/") + ".png";
		URL imgURL = x.getResource(path);
		InputStream resource = x.getResourceAsStream(path);
		if (resource == null) {
			LOG.err(TT + ".copyImage(img=" + img + ", dir=" + dir + ")" + "unable to found " + path);
			return;
		}
		try {
			InputStream in = resource;
			try (OutputStream writer = new BufferedOutputStream(new FileOutputStream(out))) {
				byte[] bx = new byte[1024];
				int length;
				while ((length = in.read(bx)) >= 0) {
					writer.write(bx, 0, length);
				}
			}
		} catch (IOException ex) {
			LOG.err("ExportBookHtml.copyImage " + ex.getLocalizedMessage());
		}
	}

	/**
	 * get the Chapter file name
	 *
	 * @param chapter
	 * @return
	 */
	private String chapterGetFilename(Chapter chapter) {
		return (String.format("%s-C%02d.html", book.getTitle(), chapter.getId()));
	}

	/**
	 * get the Scene file name
	 *
	 * @param scene
	 * @return
	 */
	private String sceneGetFilename(Scene scene) {
		if (!paramBook.getHtmlMultiChapter() && !paramBook.getHtmlMultiScene()) {
			return ("");
		}
		return Scene.getFilename(scene, book.getTitle());
	}

}
