/*
 * Copyright (C) 2023 favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.exim.exporter;

import i18n.I18N;
import java.util.List;
import storybook.model.EntityUtil;
import storybook.model.hbn.entity.Chapter;
import storybook.model.hbn.entity.Endnote;
import storybook.model.hbn.entity.Part;
import storybook.model.hbn.entity.Scene;
import storybook.toolkit.LOG;
import storybook.toolkit.file.IOUtil;
import storybook.toolkit.html.Html;
import storybook.ui.MainFrame;
import storybook.ui.panel.typist.CommentsPanel;

/**
 *
 * @author favdb
 */
public class ExportComments extends AbstractExport {

	private static final String TT = "ExportComments";

	private static final String EXPORT_TITLE = "Comments Export",
			MSG_SCENE_MISSING = I18N.getMsg("comment.missing_scene"),
			MSG_CHAPTER_MISSING = I18N.getMsg("comment.missing_chapter"),
			MSG_PART_MISSING = I18N.getMsg("comment.missing_part");
	private CommentsPanel commentsPanel;
	private boolean titlePart = false;
	private Part curPart = null;
	private Chapter curChapter = null;
	private Scene curScene = null;

	public ExportComments(MainFrame mainFrame, String format) {
		super(mainFrame, format);
	}

	public static boolean exec(MainFrame mainFrame, CommentsPanel commentsPanel, String format) {
		LOG.trace(TT + ".exec(mainFrame, commentsPanel, format=" + format + ")");
		ExportComments exp = new ExportComments(mainFrame, format);
		exp.commentsPanel = commentsPanel;
		return exp.exec();
	}

	private boolean exec() {
		LOG.trace(TT + ".exec()");
		boolean rc = false;
		if (commentsPanel != null) {
			titlePart = EntityUtil.findParts(mainFrame).size() > 1;
			String fname = mainFrame.getH2File().getName() + "_Comments";
			fname = IOUtil.filenameCleanup(fname);
			if (openFile(fname)) {
				writeText(getComments());
				closeFile(true);
				rc = true;
			}
		}
		return rc;
	}

	@Override
	public boolean openFile(String name) {
		LOG.trace(TT + ".open(name=\"" + name + "\")");
		return super.openFile(name);
	}

	private String getComments() {
		LOG.trace(TT + ".getComments() format=" + param.getFormat());
		StringBuilder b = new StringBuilder();
		@SuppressWarnings("unchecked")
		List<Endnote> endnotes = Endnote.find(mainFrame, 1);
		for (Endnote en : endnotes) {
			b.append(getComment(en));
		}
		return b.toString();
	}

	private String getComment(Endnote en) {
		LOG.trace(TT + ".getComment(en=" + LOG.traceEntity(en)
				+ ") format=" + param.getFormat());
		StringBuilder b = new StringBuilder();
		b.append(getSceneTitle(en));
		switch (param.getFormat()) {
			case F_HTML:
				b.append(Html.intoP("[" + en.getNumber().toString() + "] "));
				b.append(en.getNotes());
				break;
			case F_TXT:
				b.append("[").append(en.getNumber().toString()).append("] ");
				b.append(Html.htmlToText(en.getNotes(), true)).append("\n");
				break;
			case F_XML:
				b.append(en.toXml());
				break;
		}
		return b.toString();
	}

	private String getSceneTitle(Endnote en) {
		if (en.getScene() == null) {
			return MSG_SCENE_MISSING;
		}
		StringBuilder b = new StringBuilder();
		b.append(getChapterTitle(en.getScene()));
		if (curScene == null || !curScene.equals(en.getScene())) {
			curScene = en.getScene();
			switch (param.getFormat()) {
				case F_HTML:
					b.append(Html.intoH(3, curScene.getName()));
					break;
				case F_TXT:
					b.append(curScene.getName()).append("\n");
					break;
				default:
					return "";
			}
		}
		return b.toString();
	}

	private String getChapterTitle(Scene scene) {
		Chapter chapter = scene.getChapter();
		if (chapter == null) {
			return MSG_CHAPTER_MISSING;
		}
		StringBuilder b = new StringBuilder();
		b.append(getPartTitle(chapter));
		if (curChapter == null || !curChapter.equals(chapter)) {
			curChapter = chapter;
			switch (param.getFormat()) {
				case F_HTML:
					b.append(Html.intoH(2, curChapter.getName()));
					break;
				case F_TXT:
					b.append(curChapter.getName()).append("\n");
					break;
				default:
					return "";
			}
		}
		return b.toString();
	}

	private String getPartTitle(Chapter chapter) {
		if (!titlePart) {
			return "";
		}
		Part part = chapter.getPart();
		if (!chapter.hasPart()) {
			return MSG_PART_MISSING;
		}
		StringBuilder b = new StringBuilder();
		if (curPart == null || !curPart.equals(part)) {
			curPart = part;
			switch (param.getFormat()) {
				case F_HTML:
					b.append(Html.intoH(1, curPart.getName()));
					break;
				case F_TXT:
					b.append(curPart.getName()).append("\n");
					break;
				default:
					return "";
			}
		}
		return b.toString();
	}

}
